-------------------------------------------------------------------------------
-- $Id:: EMIF_tester_top.vhd 454 2013-07-11 11:06:08Z mostafa.afgani          $
--
-- Company:          PureVLC Ltd
-- Engineer: 
-- 
-- Create Date:      15:48:12 03/15/2013 
-- Design Name:      EMIF bus tester
-- Module Name:      EMIF_tester_top - structural 
-- Project Name:     EMIF_tester
-- Target Devices:   XC6SLX45
-- Tool versions:    13.2
-- Description:      Top-level of EMIF bus tester
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;
use ieee.std_logic_unsigned.all;


library UNISIM;
use UNISIM.VComponents.all;

entity EMIF_tester_top is
   port (
      i_ema_clk    : in     STD_LOGIC;  -- ~100M EMIFA clock
      ema_we_n     : in     STD_LOGIC;
      ema_oe_n     : in     STD_LOGIC;
      ema_cs2_n    : in     STD_LOGIC;
      ema_cs3_n    : in     STD_LOGIC;
      ema_cs4_n    : in     STD_LOGIC;
      ema_cs5_n    : in     STD_LOGIC;
		dsp_en       : in     STD_LOGIC;
      rx_pae_n     : out    STD_LOGIC;
      rx_paf_n     : out    STD_LOGIC;
      rx_hf_n      : out    STD_LOGIC;
      tx_pae_n     : out    STD_LOGIC;
      tx_paf_n     : out    STD_LOGIC;
      tx_hf_n      : out    STD_LOGIC;
--      o_nmi_n      : out    STD_LOGIC;
--      io_ema_wait_n: in     STD_LOGIC_VECTOR ( 1 downto 0);
      io_ema_wait_n: out    STD_LOGIC_VECTOR ( 1 downto 0);
--      o_int        : out    STD_LOGIC_VECTOR ( 1 downto 0);
      io_ema_d     : inout  STD_LOGIC_VECTOR (15 downto 0)
);
end EMIF_tester_top;


architecture structural of EMIF_tester_top is

   component ema_dcm
      port(
         CLK_IN1  : in     std_logic; -- 100 MHz
         CLK_OUT1 : out    std_logic; -- 1X,  0 deg
         RESET    : in     std_logic;
         LOCKED   : out    std_logic);
   end component;


   component ldk_icon
      port(
         control0 : inout std_logic_vector(35 downto 0));
   end component;

   component ldk_ila
      PORT (
         control  : inout  std_logic_vector(35 downto 0);
         clk      : in     std_logic;
         trig0    : in     std_logic_vector(11 downto 0));
   end component;

   signal ema_clk       : std_logic;
   signal dcm0_locked   : std_logic;
   signal dcm0_reset    : std_logic := '0';
   signal tx_hf_n_sig   : std_logic := '1';
   signal rx_hf_n_sig   : std_logic := '1';
   signal wait_n_sig    : std_logic_vector ( 1 downto 0);
   signal ema_do        : std_logic_vector (15 downto 0);
   signal ema_di        : std_logic_vector (15 downto 0) := (others => '0');
   signal control0      : std_logic_vector (35 downto 0);
   signal trigger0      : std_logic_vector (11 downto 0);

begin

   ema_dcm0 : ema_dcm
   port map (
      clk_in1  => i_ema_clk,
      clk_out1 => ema_clk,
      reset    => dcm0_reset,
      locked   => dcm0_locked);
      dcm0_reset <= not dsp_en;

   -- Bi-directional I/O for EMA_D
   io_ema_d <= ema_do when (ema_oe_n='0' and ema_cs5_n='0') else (others => 'Z');
	ema_di   <= io_ema_d;
   
   -- ChipScope ICON
   ldk_icon0 : ldk_icon
   port map (
      control0 => control0);
      
   -- ChipScope ILA
   ldk_ila0 : ldk_ila
   port map (
      control => control0,
      clk     => ema_clk,
      trig0   => trigger0);
      
--   o_nmi_n      <= "1" when (ema_cs4_n='0' or  ema_cs5_n='0') else (others => 'Z');
--   o_int        <= "00" when (ema_cs4_n='0' or  ema_cs5_n='0') else (others => 'Z');

   rx_pae_n <= '1';  -- RX FIFO is never empty
   rx_paf_n <= '0';  -- RX FIFO is almost full
   tx_pae_n <= '0';  -- TX FIFO is almost empty
   tx_paf_n <= '1';  -- TX FIFO is never full
   
   ema_do   <= x"00FF";
   
   -- Periodically toggle RX and TX FIFO half full signals to force
   --  reads on CS5 and writes on CS4 from the DSP
   process (ema_clk)
      variable txcntr: std_logic_vector(12 downto 0):= "0" & x"000";
      variable rxcntr: std_logic_vector(12 downto 0):= "0" & x"0FF";
   begin
      if (ema_clk'event and ema_clk = '1') then
         if dcm0_locked = '1' then
            txcntr := txcntr + 1;
            rxcntr := rxcntr + 1;
            if (txcntr > ("0" & x"000") and txcntr < ("0" & x"0FF")) then
               tx_hf_n_sig <= '0';
            else
               tx_hf_n_sig <= '1';
            end if;
            if (rxcntr > ("0" & x"000") and rxcntr < ("0" & x"0FF")) then
               rx_hf_n_sig <= '0';
            else
               rx_hf_n_sig <= '1';
            end if;
         end if;
      end if;
   end process;
   
   tx_hf_n <= tx_hf_n_sig;
   rx_hf_n <= rx_hf_n_sig;

--   io_ema_wait_n <= "00" when (ema_cs4_n='0' or ema_cs5_n='0') else (others=>'Z');
   io_ema_wait_n <= "ZZ";
--   wait_n_sig <= io_ema_wait_n;
   wait_n_sig <= "11";
   
   trigger0 <= ema_di(1 downto 0)
               & tx_hf_n_sig
               & rx_hf_n_sig
               & ema_cs2_n
               & ema_cs3_n
               & ema_cs4_n
               & ema_cs5_n
               & ema_oe_n
               & ema_we_n
               & wait_n_sig(1)
               & wait_n_sig(0);

 end structural;
